library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity comm_rx is
	generic (
		--time_cnt_MAX		:	std_logic_vector(8 downto 0)	:=	conv_std_logic_vector(174,9);	--20MHz / 115,200bps = 173.61
		--time_cnt_MAX_HALF	:	std_logic_vector(8 downto 0)	:=	conv_std_logic_vector(87,9)		--20MHz / 115,200bps / 2 = 86.81
		time_cnt_MAX		:	std_logic_vector(8 downto 0)	:=	conv_std_logic_vector(434,9);	-- 50,000,000Hz / 115,200bps * 1 = 434.0278  434:115,207.37bps
		time_cnt_MAX_HALF	:	std_logic_vector(8 downto 0)	:=	conv_std_logic_vector(217,9)	-- 50,000,000Hz / 115,200bps / 2 = 217.0138
	);
	port (
		srst				:	in	std_logic;
		clk					:	in	std_logic;
		
		busy				:	out	std_logic;
		
		sdat				:	in	std_logic;
		
		pdat				:	out	std_logic_vector(7 downto 0);
		pdat_parity_error	:	out	std_logic;
		pdat_valid			:	out	std_logic;
		
		debug_sdat_latch	:	out	std_logic
	);
end comm_rx;

architecture RTL of comm_rx is

	signal	initialized			:	std_logic;
	signal	state				:	std_logic_vector(3 downto 0);
	signal	time_cnt			:	std_logic_vector(8 downto 0);
	signal	flag_time_cnt_max	:	std_logic;
	signal	flag_time_cnt_max_inc	:	std_logic;	
	signal	recv_dat			:	std_logic_vector(10 downto 2);
	signal	n_pdat				:	std_logic_vector(7 downto 0);
	
	signal	last				:	std_logic;
	signal	last_d1				:	std_logic;
	
	signal	even_parity			:	std_logic;
	signal	even_parity_error	:	std_logic;
	signal	pdat_parity			:	std_logic;
	
begin
	
	process(clk)
	begin
		if rising_edge(clk) then
			if (srst = '1') then
				initialized	<=	'0';
			else
				if (sdat = '1') then
					initialized	<=	'1';
				end if;
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if rising_edge(clk) then
			if (srst = '1') then
				state	<=	"0000";
			else
				if (initialized = '1') then
					if (state = "0000") then
						if (sdat = '0') then  --start pls
							state	<=	"0001";
						end if;
					else
						if ( (flag_time_cnt_max = '1') or (flag_time_cnt_max_inc = '1') ) then
							if (state = "1010") then
								if (sdat = '1') then
									state	<=	"0000";
								end if;
							else
								state	<=	state + '1';
							end if;
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;
	
	flag_time_cnt_max     <= '1' when (time_cnt = time_cnt_MAX) else '0';
	flag_time_cnt_max_inc <= '1' when (time_cnt = (time_cnt_MAX + '1') ) else '0';
	
	process(clk)
	begin
		if rising_edge(clk) then
			if (srst = '1') then
				time_cnt	<=	(others => '0');
			else
				if (state = "0000") then
					time_cnt	<=	time_cnt_MAX_HALF;
				else
					if (flag_time_cnt_max_inc = '0') then
						if (flag_time_cnt_max = '1') then
							if (state = "1010") then
								time_cnt	<=	time_cnt + '1';
							else
								time_cnt	<=	(others => '0');
							end if;
						else
							time_cnt	<=	time_cnt + '1';
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if rising_edge(clk) then
			if (srst = '1') then
				recv_dat	<=	(others => '0');
			else
				if (flag_time_cnt_max = '1') then
					if (state /= "0001") then
						recv_dat(conv_integer(state))	<=	sdat;
					end if;
				end if;
			end if;
		end if;
	end process;
	
	last	<=	'1' when ( (flag_time_cnt_max = '1') and (state = "1010") ) else '0';
	
	process(clk)
	begin
		if rising_edge(clk) then
			if (srst = '1') then
				last_d1	<=	'0';
			else
				last_d1	<=	last;
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if rising_edge(clk) then
			if (srst = '1') then
				pdat_valid	<=	'0';
			else
				if ( last_d1 = '1' ) then
					pdat_valid	<=	'1';
				else
					pdat_valid	<=	'0';
				end if;
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if rising_edge(clk) then
			if (srst = '1') then
				pdat_parity_error	<=	'0';
			else
				if ( last_d1 = '1' ) then
					if (even_parity_error = '1') then
						pdat_parity_error	<=	'1';
					else
						pdat_parity_error	<=	'0';
					end if;
				end if;
			end if;
		end if;
	end process;
	
	busy <= '0' when (state = "0000") else '1';
	
	n_pdat(7 downto 0)	<=	recv_dat(9 downto 2);
	pdat				<=	n_pdat;
	pdat_parity			<=	recv_dat(10);
	
	even_parity	<=	n_pdat(7) xor
					n_pdat(6) xor
					n_pdat(5) xor
					n_pdat(4) xor
					n_pdat(3) xor
					n_pdat(2) xor
					n_pdat(1) xor
					n_pdat(0);
	
	even_parity_error	<=	even_parity xor pdat_parity;
	
	debug_sdat_latch	<=	flag_time_cnt_max;
	
end RTL;
