library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity comm_rx_mult_uart is
	generic (
		COMM_RX_SPEED		:	std_logic_vector(8 downto 0) := conv_std_logic_vector(434,9); -- 50,000,000Hz / 115,200bps * 1 = 434.0278  434:115,207.37bps
		COMM_RX_SPEED_HALF	:	std_logic_vector(8 downto 0) := conv_std_logic_vector(217,9)  -- 50,000,000Hz / 115,200bps / 2 = 217.0138  217:115,207.37bps
	);
	port (
		srst	:	in	std_logic;
		clk		:	in	std_logic;
		
		busy	:	out	std_logic;
		--pdatmax	:	in	std_logic_vector(4 downto 0);
		
		sdat	:	in	std_logic;
		
		pdat_parity_error	:	out	std_logic;
		--pdat_chksum_error	:	out	std_logic;
		pdat_valid			:	out	std_logic;
		pdat_cnt_max		:	out	std_logic_vector(4 downto 0);
		
		--pdat_chksum			:	out	std_logic_vector(7 downto 0);
		pdat00				:	out	std_logic_vector(7 downto 0);
		pdat01				:	out	std_logic_vector(7 downto 0);
		pdat02				:	out	std_logic_vector(7 downto 0);
		pdat03				:	out	std_logic_vector(7 downto 0);
		pdat04				:	out	std_logic_vector(7 downto 0);
		pdat05				:	out	std_logic_vector(7 downto 0);
		pdat06				:	out	std_logic_vector(7 downto 0);
		pdat07				:	out	std_logic_vector(7 downto 0);
		pdat08				:	out	std_logic_vector(7 downto 0);
		pdat09				:	out	std_logic_vector(7 downto 0);
		pdat10				:	out	std_logic_vector(7 downto 0);
		pdat11				:	out	std_logic_vector(7 downto 0);
		pdat12				:	out	std_logic_vector(7 downto 0);
		pdat13				:	out	std_logic_vector(7 downto 0);
		pdat14				:	out	std_logic_vector(7 downto 0);
		pdat15				:	out	std_logic_vector(7 downto 0);
		pdat16				:	out	std_logic_vector(7 downto 0);
		pdat17				:	out	std_logic_vector(7 downto 0);
		pdat18				:	out	std_logic_vector(7 downto 0);
		pdat19				:	out	std_logic_vector(7 downto 0);
		pdat20				:	out	std_logic_vector(7 downto 0);
		pdat21				:	out	std_logic_vector(7 downto 0);
		pdat22				:	out	std_logic_vector(7 downto 0);
		pdat23				:	out	std_logic_vector(7 downto 0);
		pdat24				:	out	std_logic_vector(7 downto 0);
		pdat25				:	out	std_logic_vector(7 downto 0);
		pdat26				:	out	std_logic_vector(7 downto 0);
		pdat27				:	out	std_logic_vector(7 downto 0);
		pdat28				:	out	std_logic_vector(7 downto 0);
		pdat29				:	out	std_logic_vector(7 downto 0);
		pdat30				:	out	std_logic_vector(7 downto 0);
		pdat31				:	out	std_logic_vector(7 downto 0)
	);
end comm_rx_mult_uart;
	
architecture RTL of comm_rx_mult_uart is
	
	constant	pdatmax	:	std_logic_vector(4 downto 0) := "11111";
	
	constant	ASCII_CR	:	std_logic_vector(7 downto 0) := X"0D";
	constant	ASCII_LF	:	std_logic_vector(7 downto 0) := X"0A";
	
	
	component comm_rx is
		generic (
			time_cnt_MAX		:	std_logic_vector(8 downto 0)	:=	conv_std_logic_vector(434,9);	-- 50,000,000Hz / 115,200bps * 1 = 434.0278  434:115,207.37bps
			time_cnt_MAX_HALF	:	std_logic_vector(8 downto 0)	:=	conv_std_logic_vector(217,9)	-- 50,000,000Hz / 115,200bps / 2 = 217.0138
		);
		port (
			srst				:	in	std_logic;
			clk					:	in	std_logic;
			
			busy				:	out	std_logic;
			
			sdat				:	in	std_logic;
			
			pdat				:	out	std_logic_vector(7 downto 0);
			pdat_parity_error	:	out	std_logic;
			pdat_valid			:	out	std_logic;
			
			debug_sdat_latch	:	out	std_logic
		);
	end component comm_rx;
	
	component checksum is
		port (
			srst	:	in	std_logic;
			clk		:	in	std_logic;
			
			d		:	in	std_logic_vector(7 downto 0);
			d_valid	:	in	std_logic;
			
			q		:	out	std_logic_vector(7 downto 0);
			q_valid	:	out	std_logic
		);
	end component checksum;
	
	signal	c_busy					:	std_logic;
	signal	c_sdat					:	std_logic;
	signal	c_pdat					:	std_logic_vector(7 downto 0);
	signal	c_pdat_d1				:	std_logic_vector(7 downto 0);
	signal	c_pdat_parity_error		:	std_logic;
	signal	c_pdat_valid			:	std_logic;
	
	type	state_t	is	( IDLE, RECVWAIT, CHKSUMISSUE );
	signal	state					:	state_t;
	
	signal	pdat_cnt				:	std_logic_vector(4 downto 0);
	signal	flag_pdat_cnt_max		:	std_logic;
	--signal	c_pdat_latch		:	std_logic_vector(7 downto 0);
	
	signal	checksum_srst			:	std_logic;
	signal	checksum_d_valid		:	std_logic;
	signal	checksum_q				:	std_logic_vector(7 downto 0);
	signal	checksum_q_valid		:	std_logic;
	
	
	signal	debug_sdat_latch	:	std_logic;
	
begin
	
	c_sdat	<=	sdat;
	
	u_comm_rx: comm_rx
		generic map(
			time_cnt_MAX		=>	COMM_RX_SPEED,
			time_cnt_MAX_HALF	=>	COMM_RX_SPEED_HALF
		)
		port map(
			srst				=>	srst,
			clk					=>	clk,
			
			busy				=>	c_busy,
			sdat				=>	c_sdat,
			
			pdat				=>	c_pdat,
			pdat_parity_error	=>	c_pdat_parity_error,
			pdat_valid			=>	c_pdat_valid,
			
			--debug_sdat_latch	=>	open
			debug_sdat_latch	=>	debug_sdat_latch
		);
	
	flag_pdat_cnt_max		<=	'1' when (pdat_cnt = pdatmax) else '0';
	
		
	process(clk)
	begin
		if rising_edge(clk) then
			if (srst = '1') then
				state				<=	IDLE;
				pdat_cnt			<=	(others => '0');
				busy				<=	'0';
				pdat_parity_error	<=	'0';
				--pdat_chksum_error	<=	'0';
				pdat_valid			<=	'0';
				c_pdat_d1			<=	(others => '0');
				pdat_cnt_max		<=	(others => '0');
				pdat00				<=	(others => '0');
				pdat01				<=	(others => '0');
				pdat02				<=	(others => '0');
				pdat03				<=	(others => '0');
				pdat04				<=	(others => '0');
				pdat05				<=	(others => '0');
				pdat06				<=	(others => '0');
				pdat07				<=	(others => '0');
				pdat08				<=	(others => '0');
				pdat09				<=	(others => '0');
				pdat10				<=	(others => '0');
				pdat11				<=	(others => '0');
				pdat12				<=	(others => '0');
				pdat13				<=	(others => '0');
				pdat14				<=	(others => '0');
				pdat15				<=	(others => '0');
				pdat16				<=	(others => '0');
				pdat17				<=	(others => '0');
				pdat18				<=	(others => '0');
				pdat19				<=	(others => '0');
				pdat20				<=	(others => '0');
				pdat21				<=	(others => '0');
				pdat22				<=	(others => '0');
				pdat23				<=	(others => '0');
				pdat24				<=	(others => '0');
				pdat25				<=	(others => '0');
				pdat26				<=	(others => '0');
				pdat27				<=	(others => '0');
				pdat28				<=	(others => '0');
				pdat29				<=	(others => '0');
				pdat30				<=	(others => '0');
				pdat31				<=	(others => '0');
			else
				case state is
					when IDLE   	=>
									pdat_valid			<=	'0';
									if (c_pdat_valid = '1') then
										state				<=	RECVWAIT;
										pdat_cnt			<=	"00001";
										pdat00				<=	c_pdat;
										c_pdat_d1			<=	c_pdat;
										pdat_parity_error	<=	c_pdat_parity_error;
										busy				<=	'1';
									else
										pdat_cnt			<=	"00000";
									end if;
					when RECVWAIT	=>
									if (c_pdat_valid = '1') then
										
										case (pdat_cnt) is
											when "00001" => pdat01 <= c_pdat;
											when "00010" => pdat02 <= c_pdat;
											when "00011" => pdat03 <= c_pdat;
											when "00100" => pdat04 <= c_pdat;
											when "00101" => pdat05 <= c_pdat;
											when "00110" => pdat06 <= c_pdat;
											when "00111" => pdat07 <= c_pdat;
											when "01000" => pdat08 <= c_pdat;
											when "01001" => pdat09 <= c_pdat;
											when "01010" => pdat10 <= c_pdat;
											when "01011" => pdat11 <= c_pdat;
											when "01100" => pdat12 <= c_pdat;
											when "01101" => pdat13 <= c_pdat;
											when "01110" => pdat14 <= c_pdat;
											when "01111" => pdat15 <= c_pdat;
											when "10000" => pdat16 <= c_pdat;
											when "10001" => pdat17 <= c_pdat;
											when "10010" => pdat18 <= c_pdat;
											when "10011" => pdat19 <= c_pdat;
											when "10100" => pdat20 <= c_pdat;
											when "10101" => pdat21 <= c_pdat;
											when "10110" => pdat22 <= c_pdat;
											when "10111" => pdat23 <= c_pdat;
											when "11000" => pdat24 <= c_pdat;
											when "11001" => pdat25 <= c_pdat;
											when "11010" => pdat26 <= c_pdat;
											when "11011" => pdat27 <= c_pdat;
											when "11100" => pdat28 <= c_pdat;
											when "11101" => pdat29 <= c_pdat;
											when "11110" => pdat30 <= c_pdat;
											when "11111" => pdat31 <= c_pdat;
											when others  => null;
										end case;
										
										--c_pdat_latch	<=	c_pdat;
										if (c_pdat_parity_error = '1') then
											pdat_parity_error	<=	'1';
										end if;
										
										if ( (flag_pdat_cnt_max = '1') or ( (c_pdat_d1 = ASCII_CR) and (c_pdat = ASCII_LF) ) ) then
											state	<=	CHKSUMISSUE;
										else
											pdat_cnt	<=	pdat_cnt + '1';
										end if;
										
										c_pdat_d1		<=	c_pdat;
										
									end if;
					when CHKSUMISSUE	=>
										--pdat_chksum	<=	checksum_q;
										pdat_valid		<=	'1';
										pdat_cnt_max	<=	pdat_cnt;
										--if (checksum_q = c_pdat_latch) then
										--	pdat_chksum_error	<=	'0';
										--else
										--	pdat_chksum_error	<=	'1';
										--end if;
										busy		<=	'0';
										state		<=	IDLE;
					when others 		=>
										state		<=	IDLE;
				end case;
			end if;
		end if;
	end process;
	
	checksum_srst		<=	'1'	when	( (srst = '1') or ( (state = IDLE) and (c_pdat_valid = '0') ) ) else '0';
	checksum_d_valid	<=	'0'	when	(flag_pdat_cnt_max = '1') else c_pdat_valid;
	
	uut_checksum: checksum
		port map(
			srst	=>	checksum_srst,
			clk		=>	clk			,
			
			d		=>	c_pdat		,
			d_valid	=>	checksum_d_valid,
			
			q		=>	checksum_q,
			q_valid	=>	checksum_q_valid
		);
	
	
end RTL;
