library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity comm_tx is
	port (
		srst	:	in	std_logic;
		clk		:	in	std_logic;
		clk_en	:	in	std_logic;
		
		start	:	in	std_logic;
		busy	:	out	std_logic;
		
		pdat	:	in	std_logic_vector(7 downto 0);
		sdat	:	out	std_logic
	);
end comm_tx;
	
architecture RTL of comm_tx is
	
	constant	STARTBIT	:	std_logic	:=	'0';
	constant	STOPBIT		:	std_logic	:=	'1';
	
	signal		state		:	std_logic_vector(3 downto 0);
	signal		r_pdat		:	std_logic_vector(1 to 10);
	
	signal		even_parity	:	std_logic;
	
begin
	
	even_parity	<=	pdat(7) xor
					pdat(6) xor
					pdat(5) xor
					pdat(4) xor
					pdat(3) xor
					pdat(2) xor
					pdat(1) xor
					pdat(0);
	
	process (clk)
	begin
		if rising_edge(clk) then
			if (srst = '1') then
				state	<=	"0000";
			else
				if (clk_en = '1') then
					if (state = "0000") then
						if (start = '1') then
							state	<=	"0001";
						end if;
					else
						if (state = "1010") then
							state	<=	"0000";
						else
							state	<=	state + '1';
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;
	
	process (clk)
	begin
		if rising_edge(clk) then
			if ( srst = '1') then
				sdat	<=	STOPBIT;
			else
				if (clk_en = '1') then
					if (state = "0000") then
						if (start = '1') then
							sdat	<=	STARTBIT;
						end if;
					else
							sdat	<=	r_pdat(conv_integer(state));
					end if;
				end if;
			end if;
		end if;
	end process;
	
	process (clk)
	begin
		if rising_edge(clk) then
			if (srst = '1') then
				r_pdat(1 to 10)	<=	( "00000000" & '0' & STOPBIT);
			else
				if (clk_en = '1') then
					if (state = "0000") then
						if (start = '1') then
							r_pdat(1 to 10)	<=	( 	pdat(0) &
													pdat(1) &
													pdat(2) &
													pdat(3) &
													pdat(4) &
													pdat(5) &
													pdat(6) &
													pdat(7) &
													even_parity & STOPBIT );
						end if;
					end if;
				end if;
			end if;
		end if;
	end process;
	
	busy <= '0' when (state = "0000") else '1';
	
end RTL;
