library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity comm_tx_mult is
	generic (
		COMM_TX_SPEED	:	std_logic_vector(8 downto 0) := conv_std_logic_vector(434,9); -- 50,000,000Hz / 115,200bps * 1 = 434.0278  434:115,207.37bps
		CHECKSUM_ENABLE	:	boolean	:=	FALSE
	);
	port (
		srst	:	in	std_logic;
		clk		:	in	std_logic;
		
		start	:	in	std_logic;
		busy	:	out	std_logic;
		pdatmax	:	in	std_logic_vector(4 downto 0);
		
		pdat00	:	in	std_logic_vector(7 downto 0);
		pdat01	:	in	std_logic_vector(7 downto 0);
		pdat02	:	in	std_logic_vector(7 downto 0);
		pdat03	:	in	std_logic_vector(7 downto 0);
		pdat04	:	in	std_logic_vector(7 downto 0);
		pdat05	:	in	std_logic_vector(7 downto 0);
		pdat06	:	in	std_logic_vector(7 downto 0);
		pdat07	:	in	std_logic_vector(7 downto 0);
		pdat08	:	in	std_logic_vector(7 downto 0);
		pdat09	:	in	std_logic_vector(7 downto 0);
		pdat10	:	in	std_logic_vector(7 downto 0);
		pdat11	:	in	std_logic_vector(7 downto 0);
		pdat12	:	in	std_logic_vector(7 downto 0);
		pdat13	:	in	std_logic_vector(7 downto 0);
		pdat14	:	in	std_logic_vector(7 downto 0);
		pdat15	:	in	std_logic_vector(7 downto 0);
		pdat16	:	in	std_logic_vector(7 downto 0);
		pdat17	:	in	std_logic_vector(7 downto 0);
		pdat18	:	in	std_logic_vector(7 downto 0);
		pdat19	:	in	std_logic_vector(7 downto 0);
		pdat20	:	in	std_logic_vector(7 downto 0);
		pdat21	:	in	std_logic_vector(7 downto 0);
		pdat22	:	in	std_logic_vector(7 downto 0);
		pdat23	:	in	std_logic_vector(7 downto 0);
		pdat24	:	in	std_logic_vector(7 downto 0);
		pdat25	:	in	std_logic_vector(7 downto 0);
		pdat26	:	in	std_logic_vector(7 downto 0);
		pdat27	:	in	std_logic_vector(7 downto 0);
		pdat28	:	in	std_logic_vector(7 downto 0);
		pdat29	:	in	std_logic_vector(7 downto 0);
		pdat30	:	in	std_logic_vector(7 downto 0);
		pdat31	:	in	std_logic_vector(7 downto 0);
		
		sdat	:	out	std_logic
	);
end comm_tx_mult;
	
architecture RTL of comm_tx_mult is
	
	component comm_clken is
		generic (
			CNTMAX	:	std_logic_vector(8 downto 0) := conv_std_logic_vector(43,9)
		);
		port	(
			srst	:	in	std_logic;
			clk		:	in	std_logic;
			en		:	out	std_logic
		);
	end component comm_clken;
	
	component comm_tx is
		port (
			srst	:	in	std_logic;
			clk		:	in	std_logic;
			clk_en	:	in	std_logic;
			
			start	:	in	std_logic;
			busy	:	out	std_logic;
			
			pdat	:	in	std_logic_vector(7 downto 0);
			sdat	:	out	std_logic
		);
	end component comm_tx;
	
	component checksum is
		port (
			srst	:	in	std_logic;
			clk		:	in	std_logic;
			
			d		:	in	std_logic_vector(7 downto 0);
			d_valid	:	in	std_logic;
			
			q		:	out	std_logic_vector(7 downto 0);
			q_valid	:	out	std_logic
		);
	end component checksum;
	
	signal	clk_en	:	std_logic;
	signal	c_start	:	std_logic;
	signal	c_busy	:	std_logic;
	signal	c_pdat	:	std_logic_vector(7 downto 0);
	signal	c_sdat	:	std_logic;
	
	type	state_t	is	( IDLE, CHECKSUMSTART, CHECKSUMDONE, SENTSTART, SENTWAIT1, SENTWAIT2, PDATCNTUPWAIT );
	signal	state	:	state_t;
	
	signal	pdat_cnt	:	std_logic_vector(4 downto 0);
	signal	flag_pdat_cnt_max		:	std_logic;
	signal	flag_pdat_cnt_checksum	:	std_logic;
	
	signal	checksum_srst		:	std_logic;
	signal	checksum_d_valid	:	std_logic;
	signal	checksum_q			:	std_logic_vector(7 downto 0);
	signal	checksum_q_valid	:	std_logic;
	
begin
	
	u_comm_clken: comm_clken
		generic map(
			CNTMAX	=>	COMM_TX_SPEED
		)
		port map(
			srst	=>	srst,
			clk		=>	clk,
			en		=>	clk_en
		);
	
	u_comm_tx: comm_tx
		--generic map(
		--	COMM_TX_SPEED	=>	COMM_TX_SPEED, -- 50,000,000Hz / 115,200bps * 1 = 434.0278  434:115,207.37bps
		--	CHECKSUM_ENABLE	=>	TRUE
		--)
		port map(
			srst	=>	srst,
			clk		=>	clk,
			clk_en	=>	clk_en,
			
			start	=>	c_start,
			busy	=>	c_busy,
			
			pdat	=>	c_pdat,
			sdat	=>	c_sdat
		);
	
	flag_pdat_cnt_max		<=	'1' when (pdat_cnt = pdatmax) else '0';
	flag_pdat_cnt_checksum	<=	'0' when (state=IDLE) else flag_pdat_cnt_max;
	
	checksum_srst		<=	'1' when ( (srst='1') or (state=IDLE) ) else '0';
	checksum_d_valid	<=	'1' when ( (CHECKSUM_ENABLE=TRUE) and (state=CHECKSUMSTART) and (flag_pdat_cnt_checksum='0') ) else '0';
	
	u_checksum: checksum
		port map(
			srst	=>	checksum_srst,
			clk		=>	clk,
			
			d		=>	c_pdat,
			d_valid	=>	checksum_d_valid,
			
			q		=>	checksum_q,
			q_valid	=>	checksum_q_valid
		);
		
	process(clk)
	begin
		if rising_edge(clk) then
			if (srst = '1') then
				state		<=	IDLE;
				pdat_cnt	<=	(others => '0');
				c_start		<=	'0';
				busy		<=	'0';
			else
				case state is
					when IDLE   	=>
									pdat_cnt	<=	(others => '0');
									if (start = '1') then
										state	<=	CHECKSUMSTART;
										busy	<=	'1';
									end if;
					when CHECKSUMSTART	=>
									state		<=	CHECKSUMDONE;
					when CHECKSUMDONE	=>
									state		<=	SENTSTART;
					when SENTSTART	=>
									if (clk_en = '1') then
										c_start	<=	'1';
										state	<=	SENTWAIT1;
									end if;
					when SENTWAIT1	=>	
									if (clk_en = '1') then
										state	<=	SENTWAIT2;
									end if;
					when SENTWAIT2	=>
									if (clk_en = '1') then
										c_start	<=	'0';
										if (c_busy	=	'0') then
											if (flag_pdat_cnt_max = '1') then
												state	<=	IDLE;
												busy	<=	'0';
											else
												state		<=	PDATCNTUPWAIT;
												pdat_cnt	<=	pdat_cnt + '1';
											end if;
										end if;
									end if;
					when PDATCNTUPWAIT	=>
									state		<=	CHECKSUMSTART;
					when others 	=>
									state <= IDLE;
				end case;
			end if;
		end if;
	end process;
	
	process(clk)
	begin
		if rising_edge(clk) then
			if (srst='1') then
				c_pdat	<=	(others => '0');
			else
				if ( (CHECKSUM_ENABLE=TRUE) and (flag_pdat_cnt_checksum='1') ) then
					c_pdat	<=	checksum_q;
				else
					case (pdat_cnt(4 downto 0)) is
						when "00000" =>	c_pdat	<=	pdat00;
						when "00001" =>	c_pdat	<=	pdat01;
						when "00010" =>	c_pdat	<=	pdat02;
						when "00011" =>	c_pdat	<=	pdat03;
						when "00100" =>	c_pdat	<=	pdat04;
						when "00101" =>	c_pdat	<=	pdat05;
						when "00110" =>	c_pdat	<=	pdat06;
						when "00111" =>	c_pdat	<=	pdat07;
						when "01000" =>	c_pdat	<=	pdat08;
						when "01001" =>	c_pdat	<=	pdat09;
						when "01010" =>	c_pdat	<=	pdat10;
						when "01011" =>	c_pdat	<=	pdat11;
						when "01100" =>	c_pdat	<=	pdat12;
						when "01101" =>	c_pdat	<=	pdat13;
						when "01110" =>	c_pdat	<=	pdat14;
						when "01111" =>	c_pdat	<=	pdat15;
						when "10000" =>	c_pdat	<=	pdat16;
						when "10001" =>	c_pdat	<=	pdat17;
						when "10010" =>	c_pdat	<=	pdat18;
						when "10011" =>	c_pdat	<=	pdat19;
						when "10100" =>	c_pdat	<=	pdat20;
						when "10101" =>	c_pdat	<=	pdat21;
						when "10110" =>	c_pdat	<=	pdat22;
						when "10111" =>	c_pdat	<=	pdat23;
						when "11000" =>	c_pdat	<=	pdat24;
						when "11001" =>	c_pdat	<=	pdat25;
						when "11010" =>	c_pdat	<=	pdat26;
						when "11011" =>	c_pdat	<=	pdat27;
						when "11100" =>	c_pdat	<=	pdat28;
						when "11101" =>	c_pdat	<=	pdat29;
						when "11110" =>	c_pdat	<=	pdat30;
						when "11111" =>	c_pdat	<=	pdat31;
						when others  =>	c_pdat	<=	(others => 'X');
					end case;
				end if;
			end if;
		end if;
	end process;
	
	sdat	<=	c_sdat;
	
end RTL;
